@@ -1,7 +1,7 @@
 use std::fmt;
 use std::io::Read;
 
-use decoder::{decode, DecodedMap};
+use decoder::{decode, decode_slice, DecodedMap};
 use errors::{Result, Error};
 
 /// Represents a raw token
@@ -64,7 +64,7 @@ impl<'a> Token<'a> {
     }
 
     /// get the source if it exists as string
-    pub fn get_source(&self) -> &str {
+    pub fn get_source(&self) -> &'a str {
         if self.raw.src_id == !0 {
             ""
         } else {
@@ -73,7 +73,7 @@ impl<'a> Token<'a> {
     }
 
     /// get the name if it exists as string
-    pub fn get_name(&self) -> Option<&str> {
+    pub fn get_name(&self) -> Option<&'a str> {
         if self.raw.name_id == !0 {
             None
         } else {
@@ -88,7 +88,7 @@ impl<'a> Token<'a> {
 
     /// Converts the token into a debug tuple in the form
     /// `(source, src_line, src_col, name)`
-    pub fn to_tuple(&self) -> (&str, u32, u32, Option<&str>) {
+    pub fn to_tuple(&self) -> (&'a str, u32, u32, Option<&'a str>) {
         (
             self.get_source(),
             self.get_src_line(),
@@ -120,6 +120,40 @@ impl<'a> Iterator for TokenIter<'a> {
     }
 }
 
+/// Iterates over all tokens in a sourcemap
+pub struct NameIter<'a> {
+    i: &'a SourceMap,
+    next_idx: u32,
+}
+
+impl<'a> Iterator for NameIter<'a> {
+    type Item = &'a str;
+
+    fn next(&mut self) -> Option<&'a str> {
+        self.i.get_name(self.next_idx).map(|name| {
+            self.next_idx += 1;
+            name
+        })
+    }
+}
+
+/// Iterates over all index items in a sourcemap
+pub struct IndexIter<'a> {
+    i: &'a SourceMap,
+    next_idx: usize,
+}
+
+impl<'a> Iterator for IndexIter<'a> {
+    type Item = (u32, u32, u32);
+
+    fn next(&mut self) -> Option<(u32, u32, u32)> {
+        self.i.index.get(self.next_idx).map(|idx| {
+            self.next_idx += 1;
+            *idx
+        })
+    }
+}
+
 impl<'a> fmt::Debug for Token<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "<Token {}>", self)
@@ -207,6 +241,28 @@ impl SourceMap {
         }
     }
 
+    /// Creates a sourcemap from a reader over a JSON byte slice in UTF-8
+    /// format.  Optionally a "garbage header" as defined by the
+    /// sourcemap draft specification is supported.  In case an indexed
+    /// sourcemap is encountered an error is returned.
+    ///
+    /// ```rust
+    /// use sourcemap::SourceMap;
+    /// let input: &[_] = b"{
+    ///     \"version\":3,
+    ///     \"sources\":[\"coolstuff.js\"],
+    ///     \"names\":[\"x\",\"alert\"],
+    ///     \"mappings\":\"AAAA,GAAIA,GAAI,EACR,IAAIA,GAAK,EAAG,CACVC,MAAM\"
+    /// }";
+    /// let sm = SourceMap::from_slice(input).unwrap();
+    /// ```
+    pub fn from_slice(slice: &[u8]) -> Result<SourceMap> {
+        match try!(decode_slice(slice)) {
+            DecodedMap::Regular(sm) => Ok(sm),
+            DecodedMap::Index(_) => Err(Error::IndexedSourcemap),
+        }
+    }
+
     /// Constructs a new sourcemap from raw components.
     ///
     /// - `version`: the version is typically `3` which is the current darft version
@@ -295,6 +351,11 @@ impl SourceMap {
             .and_then(|bucket| bucket.as_ref()).map(|x| &**x)
     }
 
+    /// Returns an iterator over the names.
+    pub fn names<'a>(&'a self) -> NameIter<'a> {
+        NameIter { i: self, next_idx: 0 }
+    }
+
     /// Returns the number of names in the sourcemap.
     pub fn get_name_count(&self) -> u32 {
         self.names.len() as u32
@@ -304,6 +365,16 @@ impl SourceMap {
     pub fn get_name(&self, idx: u32) -> Option<&str> {
         self.names.get(idx as usize).map(|x| &x[..])
     }
+
+    /// Returns the number of items in the index
+    pub fn get_index_size(&self) -> usize {
+        self.index.len()
+    }
+
+    /// Returns the number of items in the index
+    pub fn index_iter<'a>(&'a self) -> IndexIter<'a> {
+        IndexIter { i: self, next_idx: 0 }
+    }
 }
 
 impl SourceMapIndex {
@@ -319,6 +390,17 @@ impl SourceMapIndex {
         }
     }
 
+    /// Creates a sourcemap index from a reader over a JSON byte slice in UTF-8
+    /// format.  Optionally a "garbage header" as defined by the
+    /// sourcemap draft specification is supported.  In case a regular
+    /// sourcemap is encountered an error is returned.
+    pub fn from_slice(slice: &[u8]) -> Result<SourceMapIndex> {
+        match try!(decode_slice(slice)) {
+            DecodedMap::Regular(_) => Err(Error::RegularSourcemap),
+            DecodedMap::Index(smi) => Ok(smi),
+        }
+    }
+
     /// Constructs a new sourcemap index from raw components.
     ///
     /// - `version`: the version is typically `3` which is the current darft version
